# coding: utf-8

import torch
from torch.utils.data import Dataset


def get_shakespeare():
    return open("data/shakespeare.txt", "rt").read()


def preprocess(text: str) -> tuple:
    alphabet = sorted(set(text))
    letter_to_int = {let: ind for ind, let in enumerate(alphabet)}
    int_to_letter = {ind: let for ind, let in enumerate(alphabet)}
    letter_ints = [letter_to_int[letter] for letter in text]
    alphabet_size = len(alphabet)
    return int_to_letter, letter_to_int, alphabet_size, letter_ints


class ShakespeareDataset(Dataset):
    def __init__(self, seq_len: int):
        _, _, _, self.text = preprocess(get_shakespeare())
        self.x = torch.LongTensor(self.text[:-1])  # get the data
        self.y = torch.LongTensor(self.text[1:])
        self.seq_len = seq_len  # set the sequence length

    def __len__(self) -> int:
        return (
            len(self.text) - self.seq_len - 1
        )  # length of corpora minus sequence length minus shift

    def __getitem__(self, index: int) -> tuple:
        return (
            self.x[index : index + self.seq_len],
            self.x[index : index + self.seq_len],
        )  # return tuple of (sample, label)
